#include <avr/io.h>
#include <avr/delay.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define setb(port,pin)  port |= 1<<pin    // nastav bit
#define clrb(port,pin)  port &= ~(1<<pin) // nuluj bit
#define negb(port,pin)  port ^= 1<<pin    // neguj bit

#define HCTL_DATA PINA
#define HCTL_PORT PORTF
#define SEL1 5
#define SEL2 6
#define EN1  3 
#define EN2  4
#define OE   2
#define RSTX 0
#define RSTY 1
#define XY   7

#define HCTL_IPORT PORTE
#define ICHX 3
#define ICHY 2
#define ICHX_READ 7
#define ICHY_READ 6
#define DOFX 5 //data overflow X
#define DOFY 4 //data overflow Y



// ================== HCTL 2032 =====================
// **************** inicializace ************************
void hctl_init()
{
	clrb(HCTL_PORT,RSTX); //zastaveni citani osyX
	clrb(HCTL_PORT,RSTY); //zastaveni citani osyY
	setb(HCTL_PORT,OE); //enable OEN (output enable pin)=zadna data na vystupnim regiistru
	hctl_decmode(4); //rezim quad-dekoderu 4x
	setb(HCTL_PORT,RSTX); //zastaveni citani osyX
	setb(HCTL_PORT,RSTY); //zastaveni citani osyY
}
//*******************************************************


// ************* cteni z dat portu HCTL *****************
char hctl_readbyte(void) 
{
	unsigned char Prvni_s,Prvni_n;
	Prvni_s = HCTL_DATA;
	Prvni_n = HCTL_DATA;
	if (Prvni_s == Prvni_n)
		return(Prvni_n);
	else 
		hctl_readbyte();
}

//*******************************************************


// ******** vyceni 32B-registru pro danou osu************

volatile uint32_t hctl_read32(char osa)
{
uint32_t d1,d2,d3,d4;
volatile uint32_t pos;
volatile unsigned char buf[] = {0,0,0,0};

    pos = 0UL;  d1=0; d2=0; d3=0; d4=0;
	if (osa == 0) clrb(HCTL_PORT,XY); //osa X
	if (osa == 1) setb(HCTL_PORT,XY); //osa Y

	clrb(HCTL_PORT,OE); //enable OEN (output enable pin)
	_delay_us(25);

  clrb(HCTL_PORT,SEL1); //SEL1
	setb(HCTL_PORT,SEL2); //SEL2 (MSB)byte
	_delay_us(25);
	d1 = hctl_readbyte();
		
	setb(HCTL_PORT,SEL1); //SEL1
	setb(HCTL_PORT,SEL2); //SEL2 3.byte
	_delay_us(25);
	d2 = hctl_readbyte();

	clrb(HCTL_PORT,SEL1); //SEL1
	clrb(HCTL_PORT,SEL2); //SEL2 2.byte
	_delay_us(25);
	d3 = hctl_readbyte();

	setb(HCTL_PORT,SEL1); //SEL1
	clrb(HCTL_PORT,SEL2); //SEL2 (LSB)byte
	_delay_us(25);
	d4 = hctl_readbyte();

	setb(HCTL_PORT,OE); //disable OEN (output enable pin)
	
	pos = (uint32_t)d4 + (uint32_t)(d3<<8) + (uint32_t)(d2<<16) + (uint32_t)(d1<<24);
        
   return pos;


}
//*******************************************************


//************** reset citace dane osy ******************
void hctl_reset(char osa)
{
   	if (osa == 0) //osa X
	{
		clrb(HCTL_PORT,RSTX);
		setb(HCTL_PORT,RSTX);
	}
	
	if (osa == 1) //osa Y
	{
		clrb(HCTL_PORT,RSTY);
		setb(HCTL_PORT,RSTY);
	}

	if (osa == 2) //osa Y
	{
		clrb(HCTL_PORT,RSTX);
		clrb(HCTL_PORT,RSTY);
		setb(HCTL_PORT,RSTX);
		setb(HCTL_PORT,RSTY);
	}
}
//*******************************************************

//************** stop citace dane osy ******************
void hctl_stop(char osa)
{
   	if (osa == 0) //osa X
	{
		clrb(HCTL_PORT,RSTX);
	}
	
	if (osa == 1) //osa Y
	{
		clrb(HCTL_PORT,RSTY);
	}

	if (osa == 2) //osa XY
	{
		clrb(HCTL_PORT,RSTX);
		clrb(HCTL_PORT,RSTY);
	}
}
//*******************************************************

//************** start citace dane osy ******************
void hctl_start(char osa)
{
   	if (osa == 0) //osa X
	{
		setb(HCTL_PORT,RSTX);
	}
	
	if (osa == 1) //osa Y
	{
		setb(HCTL_PORT,RSTY);
	}

	if (osa == 2) //osa XY
	{
		setb(HCTL_PORT,RSTX);
		setb(HCTL_PORT,RSTY);
	}
}
//*******************************************************

//************** vybe mode dekoderu *********************
void hctl_decmode(char mode)
{
	 
	 setb(HCTL_PORT,EN1);  //EN1
	 setb(HCTL_PORT,EN2);	 //EN2  (defaultni )nastaveni rezimu 1x
    
	if (mode == 1)
	{
	  setb(HCTL_PORT,EN1);
	  setb(HCTL_PORT,EN2);  // 1x
	}
	if (mode == 2)
	{
	  clrb(HCTL_PORT,EN1);  // 2x
	  setb(HCTL_PORT,EN2);
	}
	if (mode == 4)
	{
	  setb(HCTL_PORT,EN1);  // 4x
	  clrb(HCTL_PORT,EN2);
	}
}

//*******************************************************
//************** zapinani I kanalu *********************
void hctl_ichanels(unsigned char i)
{
switch (i)
							{
								case 0: //vypni xy
									{			
										setb(HCTL_IPORT,ICHX);
										setb(HCTL_IPORT,ICHY);
									}
								break;
					
								case 1: //zapni x
									{
										clrb(HCTL_IPORT,ICHX);
										setb(HCTL_IPORT,ICHY);
									}
								break;

								case 2: //zapni y
									{
										setb(HCTL_IPORT,ICHX);
										clrb(HCTL_IPORT,ICHY);
									}
								break;

								case 3: //zapni xy
									{	
										clrb(HCTL_IPORT,ICHX);
										clrb(HCTL_IPORT,ICHY);
									}
								break;
								default:{};
							}
};							
