#include <avr/io.h>
#include <avr/delay.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define LCD_PORT		PORTC       //port LCD
#define LCD_PIN			PINC	    //pin registr 
#define LCD_INITSTRING	0B00101100  //inicializacni string
#define LCD_LIGHT		7           //pin osvetleni
#define LCD_RS			5			//pin RS
#define LCD_E			6			//pin enable
#define LCD_DATA4_MASK	0x0F	    //4B data pripojena na: 
									//0-3 0x0F 
									//4-7 0xF0 

#define lcd_lighton()	LCD_PORT |= 1<<LCD_LIGHT
#define lcd_lightoff()	LCD_PORT &= ~(1<<LCD_LIGHT)

#define lcd_setdata()	LCD_PORT |= 1<<LCD_RS
#define lcd_setcmd()	LCD_PORT &= ~(1<<LCD_RS)

#define lcd_sete()	LCD_PORT |= 1<<LCD_E
#define lcd_clre()	LCD_PORT &= ~(1<<LCD_E)



// =========================== LCD ================================
// inicialazace displeje pri prvni zapnuti
void lcd_init(void)
{

  lcd_setcmd();
  _delay_us(40);
  LCD_PORT = (LCD_INITSTRING & 0B11110000) >> 4; // poslani init pri 8b komunikaci
  lcd_write();
  
  lcd_lighton(); // zapne osvetleni

	lcd_cmd(LCD_INITSTRING); // znova init pri 4b komunikaci
 	lcd_cmd(0B00001100); // zapni LCD, no kurzor, no blink
 	lcd_clr(); // smaze LCD a kurzor na start
}

//zapinani kurzoru
void lcd_cursoron(char blink)
{
  if (blink == 0) {lcd_cmd(0B00001110);} // zapni LCD, kurzor, neblika
  else {lcd_cmd(0B00001111);}  // zapni LCD, kurzor, blika
}
// vypinani kurzoru
void lcd_cursoroff()
{
  lcd_cmd(0B00001100); // zapni LCD, kurzor, blika
}

// odeslani prikazu lcd
void lcd_cmd(unsigned char cmd)
{
     
  lcd_setcmd();
  _delay_us(40);
  LCD_PORT = (LCD_PIN & ~LCD_DATA4_MASK) | ((cmd & 0B11110000) >> 4);
  lcd_write();

  LCD_PORT = (LCD_PIN & ~LCD_DATA4_MASK) |(cmd & 0B00001111);
  lcd_write();
}
// odeslani dat lcd
void lcd_data(unsigned char data)
{
     
  lcd_setdata();
  _delay_us(40);
  LCD_PORT = (LCD_PIN & ~LCD_DATA4_MASK) | ((data & 0B11110000) >> 4);
  lcd_write();

  LCD_PORT = (LCD_PIN & ~LCD_DATA4_MASK) |(data & 0B00001111);
  lcd_write();
}
// smaze lcd
void lcd_clr()
{
   lcd_cmd(1);
   _delay_us(1600);
}

//zobrazi retezec znaku na displej
void lcd_str(const char* strn)
{
     
 while (*strn !='\0')
	{
	 
		lcd_data(*strn);
		strn +=1;
	}
}

void lcd_str2(const char* strn)
{
     
 while (*strn !='\0')
	{
	 
	//	usart_transmit_data8(*strn);
		_delay_ms(25);
		strn +=1;
	}

}
//umisti retezec znaku na pozici na displeji 
void lcd_posstr(const char* strn, unsigned char r, unsigned char c)
{
  
  lcd_goto(r,c);
  lcd_str(strn);

}
// premisti kurzor
void lcd_goto(unsigned char row, unsigned char col)
{
        unsigned char cell = 0;
        switch (row)
        {
                case 0: cell+=0; break;
                case 1: cell+=0x40; break;
                case 2: cell+=0x14; break;
                case 3: cell+=0x054; break;
        }                                     
        
        cell += col;
        lcd_cmd(128|cell);
}
// potvrzeni zapisu na lcd
void lcd_write()
{
  lcd_sete();
  _delay_us(5);
  lcd_clre();
  _delay_us(40);
}
//zobrazeni ciselne hodnoty
void lcd_prdec(long int co)
{
uint_fast32_t m,r;

r = 100000000;
	while (r!=1)
	{
	m = (co / r);
	co = co - (m*r);
	if (m!=0) 
	{
		lcd_data(m+48);
	}
	r=r/10;

	
	}

	lcd_data((co % 10)+48);

}
// zobrazeni hexadecimalni hodnoty
void lcd_prhex(long int co)
{
uint_fast32_t m,r;

r = 268435456;
	while (r!=1)
	{
	m = (co / r);
	co = co - (m*r);
	if (m < 10)
		lcd_data(m+48);
	else lcd_data(m+48+7);
	
	r=r/16;

	
	}

	lcd_data((co % 16)+48);

}

// ================================================================
